package com.ynxbd.push.entity.enums;


import lombok.ToString;
import org.springframework.util.ObjectUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.HashMap;
import java.util.Map;

/**
 * 枚举：类目模板消息参数值内容限制说明
 */
@ToString
public enum MsgParamEnum {

    THING(20, "thing", "【事物】可汉字、数字、字母或符号组合"),

    CHARACTER_STRING(32, "character_string", "【字符串】可数字、字母或符号组合"),
    TIME(20, "time", "【时间】24小时制时间格式（支持+年月日），支持填时间段，两个时间点之间用“~”符号连接【例如：15:01，或：2019年10月1日 15:01】"),
    AMOUNT(12, "amount", "【金额】1个币种符号+12位以内纯数字，可带小数，结尾可带“元” 可带小数"),
    PHONE_NUMBER(17, "phone_number", "【电话】（数字、符号） 电话号码，例：+86-0766-66888866"),
    CAR_NUMBER(8, "car_number", "【车牌】第一位与最后一位可为汉字，其余为字母或数字 例：车牌号码：粤A8Z888挂"),
    CONST(20, "const", "【常量】超过无法下发注：需枚举（需将内容提交平台审核，审核通过可下发）只能下发审核通过的字符串和空串"),

    // 锁定的参数名-----------------------
    URL(1024, "url", "跳转链接"),
    TEMPLATE_ID(100, "template_id", "跳转链接"),
    TIP(200, "tip", "模板类型"),
    TITLE(200, "title", "模板标题"),
    ;

    // 字符串限制长度
    public final int LENGTH;
    // code开头
    public final String CODE;
    // 备注说明
    public final String REMARK;

    MsgParamEnum(int LENGTH, String CODE, String REMARK) {
        this.LENGTH = LENGTH;
        this.CODE = CODE;
        this.REMARK = REMARK;
    }


    /**
     * 获取参数
     *
     * @param paramsMap 参数集
     * @param key       参数key
     * @param canNull   可以为空
     */
    public String getStrValue(Map<String, String> paramsMap, String key, boolean canNull) throws Exception {
        if (ObjectUtils.isEmpty(key)) {
            throw new Exception("参数key缺失");
        }
        String value = paramsMap.get(key);
        // 可以为空
        if (canNull) {
            if (ObjectUtils.isEmpty(value)) {
                return "";
            }
        } else {
            if (ObjectUtils.isEmpty(value)) {
                throw new Exception(key + "参数为空");
            }
        }
        // 不需要进行内容过滤的枚举
        if (!isFilterEnum()) {
            // 判断长度是否合法
            if (value.length() > LENGTH) {
                throw new Exception(key + "参数长度过长");
            }
        }
        return value;
    }

    /**
     * 字符串截取
     *
     * @param len 长度
     * @param val 字符串
     * @return str
     */
    public static String valSubstring(int len, String val) {
        if (ObjectUtils.isEmpty(val)) {
            return "";
        }
        if (val.length() > len) {
            val = val.substring(0, len - 3) + "...";
        }
        return val;
    }

    /**
     * 是否为需要进行内容过滤的枚举
     *
     * @return boolean
     */
    public boolean isFilterEnum() {
        return this == THING || this == CHARACTER_STRING;
    }

    /**
     * 获取参数
     *
     * @param paramsMap 参数集
     * @param key       参数key
     */
    public Map<String, String> getMapValue(Map<String, String> paramsMap, String key) throws Exception {
        return getMapValue(paramsMap, key, false, null);
    }

    /**
     * 获取参数
     *
     * @param paramsMap 参数集
     * @param key       参数key
     * @param color     颜色
     */
    public Map<String, String> getMapValue(Map<String, String> paramsMap, String key, String color) throws Exception {
        return getMapValue(paramsMap, key, false, color);
    }

    /**
     * 获取参数
     *
     * @param paramsMap 参数集
     * @param key       参数key
     * @param canNull   是否可以为空
     */
    public Map<String, String> getMapValue(Map<String, String> paramsMap, String key, boolean canNull, String color) throws Exception {
        if (ObjectUtils.isEmpty(key)) {
            throw new Exception("参数key缺失");
        }
        Map<String, String> keyMap = new HashMap<>();
        String value = getStrValue(paramsMap, key, canNull);
        if (this == TIME) {
            if (value.contains("：")) {
                value = value.replaceAll("：", ":");
            }
        } else if (this == AMOUNT) {
            try {
                value = new BigDecimal(value).setScale(2, RoundingMode.DOWN).toString();
            } catch (Exception e) {
                throw new Exception("[" + key + "][" + value + "] 金额异常");
            }
        } else if (isFilterEnum()) {
            // 需要进行内容过滤的枚举
            value = valSubstring(LENGTH, value);
        }
        keyMap.put("value", value);
        if (!ObjectUtils.isEmpty(color)) {
            keyMap.put("color", color);
        }
        return keyMap;
    }


    /**
     * 组织成key
     */
    public String toKey(String keyNum) throws Exception {
        if (ObjectUtils.isEmpty(keyNum)) {
            throw new Exception("keyNum为空");
        }
        return CODE + keyNum;
    }


    /**
     * 模板参数组装
     *
     * @param paramsMap 入参map
     * @param tempData  模板
     * @param paramName 参数名称
     * @param keyNum    key的编号（如：time21，入参就为21）
     */
    public String put(Map<String, String> paramsMap, Map<String, Object> tempData, String paramName, Integer keyNum) throws Exception {
        return put(paramsMap, tempData, paramName, keyNum, false, null);
    }

    /**
     * 模板参数组装
     *
     * @param paramsMap 入参map
     * @param tempData  模板
     * @param paramName 参数名称
     * @param keyNum    key的编号（如：time21，入参就为21）
     * @param canNull   是否可以为空
     * @param color     颜色
     */
    public String put(Map<String, String> paramsMap, Map<String, Object> tempData, String paramName, Integer keyNum, boolean canNull, String color) throws Exception {
        if (keyNum == null) {
            throw new Exception("keyNum is null");
        }
        String keyNumStr = String.valueOf(keyNum);
        tempData.put(this.toKey(keyNumStr), this.getMapValue(paramsMap, paramName, canNull, color));
        return getStrValue(paramsMap, paramName, canNull);
    }
}
