package com.ynxbd.push.quartz.scheduler;


import com.ynxbd.push.config.IniConfig;
import com.ynxbd.push.quartz.job.TimePushJob;
import com.ynxbd.push.quartz.job.TodayRegJob;
import com.ynxbd.push.quartz.job.WorkLoadJob;
import lombok.extern.slf4j.Slf4j;
import org.quartz.*;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Date;

/**
 * 任务启动的以及相关的调度器
 *
 * @author 李进才
 */
@Slf4j
@Component
public class AppQuartzScheduler {

    @Resource
    private Scheduler scheduler;

    /**
     * 开始执行所有任务
     */
    public void startNormalJob() throws SchedulerException {
        startNormalJob(scheduler);
        scheduler.start();
    }

    public void startCrontabJob() throws SchedulerException {
        startCrontabJob(scheduler);
        scheduler.start();
    }

    public void startWorkloadJob() throws SchedulerException{
        startWorkLoadJob(scheduler);
        scheduler.start();
    }

    public void startTodayRegJob() throws SchedulerException {
        startTodayRegJob(scheduler);
        scheduler.start();
    }

    /**
     * 获取Job信息
     */
    public String getJobInfo(String name, String group) throws SchedulerException {
        TriggerKey triggerKey = new TriggerKey(name, group);
        CronTrigger cronTrigger = (CronTrigger) scheduler.getTrigger(triggerKey);
        return String.format("time:%s,state:%s", cronTrigger.getCronExpression(),
                scheduler.getTriggerState(triggerKey).name());
    }

    /**
     * 修改某个任务的执行时间
     */
    public boolean modifyJob(String name, String group, String time) throws SchedulerException {
        Date date = null;
        TriggerKey triggerKey = new TriggerKey(name, group);
        CronTrigger cronTrigger = (CronTrigger) scheduler.getTrigger(triggerKey);
        String oldTime = cronTrigger.getCronExpression();
        if (!oldTime.equalsIgnoreCase(time)) {
            CronScheduleBuilder cronScheduleBuilder = CronScheduleBuilder.cronSchedule(time);
            CronTrigger trigger = TriggerBuilder.newTrigger().withIdentity(name, group)
                    .withSchedule(cronScheduleBuilder).build();
            date = scheduler.rescheduleJob(triggerKey, trigger);
        }
        return date != null;
    }

    /**
     * 暂停所有任务
     */
    public void pauseAllJob() throws SchedulerException {
        scheduler.pauseAll();
    }

    /**
     * 暂停某个任务
     */
    public void pauseJob(String name, String group) throws SchedulerException {
        JobKey jobKey = new JobKey(name, group);
        JobDetail jobDetail = scheduler.getJobDetail(jobKey);
        if (jobDetail == null) {
            return;
        }
        scheduler.pauseJob(jobKey);
    }

    /**
     * 恢复所有任务
     */
    public void resumeAllJob() throws SchedulerException {
        scheduler.resumeAll();
    }

    /**
     * 恢复某个任务
     */
    public void resumeJob(String name, String group) throws SchedulerException {
        JobKey jobKey = new JobKey(name, group);
        JobDetail jobDetail = scheduler.getJobDetail(jobKey);
        if (jobDetail == null) {
            return;
        }
        scheduler.resumeJob(jobKey);
    }

    /**
     * 删除某个任务
     */
    public void deleteJob(String name, String group) throws SchedulerException {
        JobKey jobKey = new JobKey(name, group);
        JobDetail jobDetail = scheduler.getJobDetail(jobKey);
        if (jobDetail == null) {
            return;
        }
        scheduler.deleteJob(jobKey);
    }

    /**
     * 简单调取器
     */
    private void startNormalJob(Scheduler scheduler) throws SchedulerException {
        JobDetail jobDetail = JobBuilder.newJob(TimePushJob.class).withIdentity("TimePush", "WorkLog").build();
        SimpleScheduleBuilder simpleScheduleBuilder = SimpleScheduleBuilder.repeatMinutelyForever();
        // 十分钟执行一次
        simpleScheduleBuilder.withIntervalInSeconds(10);
        SimpleTrigger simpleTrigger = TriggerBuilder.newTrigger().withIdentity("TimePush", "WorkLog").withSchedule(simpleScheduleBuilder).build();
        scheduler.scheduleJob(jobDetail, simpleTrigger);
    }

    /**
     * 基于 Crontab 的调度器
     */
    private void startCrontabJob(Scheduler scheduler) throws SchedulerException {
        JobDetail jobDetail = JobBuilder.newJob(TimePushJob.class).withIdentity("TimePush", "WorkLog").build();
        // 一天执行一次
        CronScheduleBuilder cronScheduleBuilder = CronScheduleBuilder.cronSchedule("0 15 18 * * ?");
        CronTrigger cronTrigger = TriggerBuilder.newTrigger().withIdentity("TimePush", "WorkLog").withSchedule(cronScheduleBuilder).build();
        scheduler.scheduleJob(jobDetail, cronTrigger);
    }

    private void startTodayRegJob(Scheduler scheduler) throws SchedulerException {
        JobDetail jobDetailMom = JobBuilder.newJob(TodayRegJob.class).withIdentity("TodayRegMom", "TodayRegMom").build();
        JobDetail jobDetailNoon = JobBuilder.newJob(TodayRegJob.class).withIdentity("TodayRegNoon", "TodayRegNoon").build();
        if(IniConfig.TODAY_REG_TIME_MOM==null||IniConfig.TODAY_REG_TIME_NOON==null){
            log.info("[定时任务]：每日预约推送未设置时间，功能未开启，todayRegTimeMom-{},todayRegTimeNoon-{}",IniConfig.TODAY_REG_TIME_MOM,IniConfig.TODAY_REG_TIME_NOON);
            return;
        }
        log.info("[定时任务]：每日预约推送未设置时间，功能已经开启todayRegTimeMom-{},todayRegTimeNoon-{}",IniConfig.TODAY_REG_TIME_MOM,IniConfig.TODAY_REG_TIME_NOON);
        CronScheduleBuilder cronScheduleBuilderMom = CronScheduleBuilder.cronSchedule(IniConfig.TODAY_REG_TIME_MOM);
        CronScheduleBuilder cronScheduleBuilderNoon = CronScheduleBuilder.cronSchedule(IniConfig.TODAY_REG_TIME_NOON);
        CronTrigger cronTriggerMom = TriggerBuilder.newTrigger().withIdentity("TodayRegMom", "TodayRegMom").withSchedule(cronScheduleBuilderMom).build();
        CronTrigger cronTriggerNoon = TriggerBuilder.newTrigger().withIdentity("TodayRegNoon", "TodayRegNoon").withSchedule(cronScheduleBuilderNoon).build();
        scheduler.scheduleJob(jobDetailMom, cronTriggerMom);
        scheduler.scheduleJob(jobDetailNoon, cronTriggerNoon);
    }

    /**
     * 基于 Crontab 的调度器
     */
    private void startWorkLoadJob(Scheduler scheduler) throws SchedulerException {
        JobDetail jobDetail = JobBuilder.newJob(WorkLoadJob.class).withIdentity("WorkLoad", "WorkLog").build();
        if(IniConfig.LEADER_TIME==null){
            log.info("[定时任务]：BI领导推送未设置时间，功能未开启");
            return;
        }
        log.info("[定时任务]：BI领导推送未设置时间，功能已经开启，推送时间-{}",IniConfig.LEADER_TIME);
        CronScheduleBuilder cronScheduleBuilder = CronScheduleBuilder.cronSchedule(IniConfig.LEADER_TIME);
        CronTrigger cronTrigger = TriggerBuilder.newTrigger().withIdentity("WorkLoad", "WorkLog").withSchedule(cronScheduleBuilder).build();
        scheduler.scheduleJob(jobDetail, cronTrigger);
    }

}
